@@ -1,17 +1,17 @@
 /*
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; You may only use version 2 of the License,
-	you have no option to use any other version.
+        This program is free software; you can redistribute it and/or modify
+        it under the terms of the GNU General Public License as published by
+        the Free Software Foundation; You may only use version 2 of the License,
+        you have no option to use any other version.
 
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
+        This program is distributed in the hope that it will be useful,
+        but WITHOUT ANY WARRANTY; without even the implied warranty of
+        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+        GNU General Public License for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        You should have received a copy of the GNU General Public License
+        along with this program; if not, write to the Free Software
+        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
         gnome theme-switcher capplet - (c) Jonathan Blandford <jrb@gnome.org>
         xfce4 mcs plugin   - (c) 2002 Olivier Fourdan
@@ -24,6 +24,13 @@
 
 #include <string.h>
 #include <gtk/gtk.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <dirent.h>
+#include <unistd.h>
+#include <string.h>
+#include <stdlib.h>
+#include <stdio.h>
 
 #include <libxfce4mcs/mcs-common.h>
 #include <libxfce4mcs/mcs-manager.h>
@@ -40,6 +47,8 @@
 #define CHANNEL "settings"
 #define PLUGIN_NAME "ui"
 #define DEFAULT_THEME "Default"
+#define INITIAL_THEME "Xfce"
+#define DEFAULT_ICON_THEME "hicolor"
 #define DEFAULT_TOOLBAR_STYLE "icons"
 #define DEFAULT_FONT "Sans 10"
 
@@ -51,9 +60,11 @@ static gboolean write_options(McsPlugin * mcs_plugin);
 static void run_dialog(McsPlugin * mcs_plugin);
 
 static gboolean setting_model = FALSE;
+static gboolean setting_itheme_model = FALSE;
 static gboolean initial_scroll = TRUE;
 static gboolean is_running = FALSE;
 static gchar *current_theme = NULL;
+static gchar *current_icon_theme = NULL;
 static gchar *current_font = NULL;
 static gchar *current_toolbar_style = NULL;
 
@@ -62,6 +73,10 @@ struct _Itf
 {
     McsPlugin *mcs_plugin;
 
+    GtkWidget *themes_notebook;
+    GtkWidget *theme_label;
+    GtkWidget *icon_theme_label;
+
     GtkWidget *theme_dialog;
     GtkWidget *dialog_vbox1;
     GtkWidget *dialog_header;
@@ -83,6 +98,11 @@ struct _Itf
     GtkWidget *menuitem2;
     GtkWidget *menuitem3;
     GtkWidget *menuitem4;
+    GtkWidget *icon_theme_treeview;
+    GtkWidget *icon_theme_swindow;
+    GtkWidget *frame4;
+    GtkWidget *vbox4;
+    GtkWidget *hbox3;
     GtkWidget *dialog_action_area1;
     GtkWidget *helpbutton1;
     GtkWidget *closebutton1;
@@ -124,6 +144,40 @@ static void theme_selection_changed(GtkTreeSelection * selection, gpointer data)
     }
 }
 
+static void icon_theme_selection_changed(GtkTreeSelection * selection, gpointer data)
+{
+    GtkTreeModel *model;
+    gchar *new_key;
+    GtkTreeIter iter;
+    McsPlugin *mcs_plugin;
+
+    if(setting_itheme_model)
+        return;
+
+    mcs_plugin = (McsPlugin *) data;
+
+    if(gtk_tree_selection_get_selected(selection, &model, &iter))
+    {
+        gtk_tree_model_get(model, &iter, THEME_NAME_COLUMN, &new_key, -1);
+    }
+    else
+    {
+        new_key = NULL;
+    }
+
+    if(new_key != NULL)
+    {
+        if(current_icon_theme && strcmp(current_icon_theme, new_key))
+        {
+            g_free(current_icon_theme);
+            current_icon_theme = new_key;
+            mcs_manager_set_string(mcs_plugin->manager, "Net/IconThemeName", CHANNEL, current_icon_theme);
+            mcs_manager_notify(mcs_plugin->manager, CHANNEL);
+            write_options(mcs_plugin);
+        }
+    }
+}
+
 static void read_themes(Itf * dialog)
 {
     static GList *gtk_theme_list = NULL;
@@ -141,7 +195,7 @@ static void read_themes(Itf * dialog)
     setting_model = TRUE;
     gtk_list_store_clear(GTK_LIST_STORE(model));
 
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dialog->theme_swindow), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dialog->theme_swindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_widget_set_size_request(dialog->theme_swindow, -1, -1);
 
     for(list = gtk_theme_list; list; list = list->next)
@@ -179,7 +233,7 @@ static void read_themes(Itf * dialog)
         GtkTreePath *path;
 
         gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter, THEME_NAME_COLUMN, current_theme, -1);
+        gtk_list_store_set(GTK_LIST_STORE(model), &iter, THEME_NAME_COLUMN, DEFAULT_THEME, -1);
 
         path = gtk_tree_model_get_path(model, &iter);
         row_ref = gtk_tree_row_reference_new(model, path);
@@ -205,6 +259,84 @@ static void read_themes(Itf * dialog)
     setting_model = FALSE;
 }
 
+static void read_icon_themes(Itf * dialog)
+{
+    static GList *icon_theme_list = NULL;
+    GList *list;
+    GtkTreeModel *model;
+    GtkTreeView *tree_view;
+    gint i;
+    gboolean current_icon_theme_found = FALSE;
+    GtkTreeRowReference *row_ref = NULL;
+
+    icon_theme_list = icon_theme_common_get_list(icon_theme_list);
+    tree_view = GTK_TREE_VIEW(dialog->icon_theme_treeview);
+    model = gtk_tree_view_get_model(tree_view);
+
+    setting_itheme_model = TRUE;
+    gtk_list_store_clear(GTK_LIST_STORE(model));
+
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dialog->icon_theme_swindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+    gtk_widget_set_size_request(dialog->icon_theme_swindow, -1, -1);
+
+    for(list = icon_theme_list; list; list = list->next)
+    {
+        ThemeInfo *info = list->data;
+        GtkTreeIter iter;
+
+        gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
+        gtk_list_store_set(GTK_LIST_STORE(model), &iter, THEME_NAME_COLUMN, info->name, -1);
+
+        if(strcmp(current_icon_theme, info->name) == 0)
+        {
+            GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
+            row_ref = gtk_tree_row_reference_new(model, path);
+            gtk_tree_path_free(path);
+            current_icon_theme_found = TRUE;
+        }
+
+        if(i == MAX_ELEMENTS_BEFORE_SCROLLING)
+        {
+            GtkRequisition rectangle;
+            gtk_widget_size_request(GTK_WIDGET(tree_view), &rectangle);
+            gtk_widget_set_size_request(dialog->icon_theme_swindow, -1, rectangle.height);
+            gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dialog->icon_theme_swindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+        }
+        i++;
+    }
+
+    if(!current_icon_theme_found)
+    {
+        GtkTreeIter iter;
+        GtkTreePath *path;
+
+        gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
+        gtk_list_store_set(GTK_LIST_STORE(model), &iter, THEME_NAME_COLUMN, DEFAULT_ICON_THEME, -1);
+
+        path = gtk_tree_model_get_path(model, &iter);
+        row_ref = gtk_tree_row_reference_new(model, path);
+        gtk_tree_path_free(path);
+    }
+
+    if(row_ref)
+    {
+        GtkTreePath *path;
+
+        path = gtk_tree_row_reference_get_path(row_ref);
+        gtk_tree_view_set_cursor(tree_view, path, NULL, FALSE);
+
+        if(initial_scroll)
+        {
+            gtk_tree_view_scroll_to_cell(tree_view, path, NULL, TRUE, 0.5, 0.0);
+            initial_scroll = FALSE;
+        }
+
+        gtk_tree_path_free(path);
+        gtk_tree_row_reference_free(row_ref);
+    }
+    setting_itheme_model = FALSE;
+}
+
 static void font_selection_ok(GtkWidget * w, gpointer user_data)
 {
     Itf *itf = (Itf *) user_data;
@@ -256,6 +388,7 @@ static gint sort_func(GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gp
 {
     gchar *a_str = NULL;
     gchar *b_str = NULL;
+    gchar *theme_name = (gchar *) user_data;
 
     gtk_tree_model_get(model, a, 0, &a_str, -1);
     gtk_tree_model_get(model, b, 0, &b_str, -1);
@@ -382,13 +515,20 @@ Itf *create_theme_dialog(McsPlugin * mcs_plugin)
     gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->hbox1, TRUE, TRUE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(dialog->hbox1), 5);
 
-    dialog->frame1 = xfce_framebox_new (_("Select theme"), FALSE);
+    dialog->themes_notebook = gtk_notebook_new();
+    gtk_notebook_set_show_border(GTK_NOTEBOOK(dialog->themes_notebook), FALSE);
+    gtk_widget_show(dialog->themes_notebook);
+    gtk_box_pack_start(GTK_BOX(dialog->hbox1), dialog->themes_notebook, TRUE, TRUE, 0);
+
+    dialog->frame1 = xfce_framebox_new(NULL, FALSE);
     gtk_widget_show(dialog->frame1);
-    gtk_box_pack_start(GTK_BOX(dialog->hbox1), dialog->frame1, TRUE, TRUE, 0);
+    dialog->theme_label = gtk_label_new_with_mnemonic(_("_Theme"));
+    gtk_widget_show(dialog->theme_label);
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->themes_notebook), dialog->frame1, dialog->theme_label);
 
     dialog->vbox2 = gtk_vbox_new(FALSE, 5);
     gtk_widget_show(dialog->vbox2);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame1), dialog->vbox2);
+    xfce_framebox_add (XFCE_FRAMEBOX(dialog->frame1), dialog->vbox2);
 
     dialog->hbox2 = gtk_hbox_new(FALSE, 8);
     gtk_widget_show(dialog->hbox2);
@@ -404,6 +544,30 @@ Itf *create_theme_dialog(McsPlugin * mcs_plugin)
     gtk_container_add(GTK_CONTAINER(dialog->theme_swindow), dialog->theme_treeview);
     gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(dialog->theme_treeview), FALSE);
 
+    dialog->frame4 = xfce_framebox_new(NULL, FALSE);
+    gtk_widget_show(dialog->frame4);
+    dialog->icon_theme_label = gtk_label_new_with_mnemonic(_("_Icon Theme"));
+    gtk_widget_show(dialog->icon_theme_label);
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->themes_notebook), dialog->frame4, dialog->icon_theme_label);
+
+    dialog->vbox4 = gtk_vbox_new(FALSE, 5);
+    gtk_widget_show(dialog->vbox4);
+    xfce_framebox_add(XFCE_FRAMEBOX(dialog->frame4), dialog->vbox4);
+
+    dialog->hbox3 = gtk_hbox_new(FALSE, 8);
+    gtk_widget_show(dialog->hbox3);
+    gtk_box_pack_start(GTK_BOX(dialog->vbox4), dialog->hbox3, TRUE, TRUE, 0);
+
+    dialog->icon_theme_swindow = gtk_scrolled_window_new(NULL, NULL);
+    gtk_widget_show(dialog->icon_theme_swindow);
+    gtk_box_pack_start(GTK_BOX(dialog->hbox3), dialog->icon_theme_swindow, TRUE, TRUE, 0);
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(dialog->icon_theme_swindow), GTK_SHADOW_IN);
+
+    dialog->icon_theme_treeview = gtk_tree_view_new();
+    gtk_widget_show(dialog->icon_theme_treeview);
+    gtk_container_add(GTK_CONTAINER(dialog->icon_theme_swindow), dialog->icon_theme_treeview);
+    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(dialog->icon_theme_treeview), FALSE);
+
     dialog->vbox3 = gtk_vbox_new(FALSE, 0);
     gtk_widget_show(dialog->vbox3);
     gtk_box_pack_start(GTK_BOX(dialog->hbox1), dialog->vbox3, TRUE, TRUE, 0);
@@ -415,7 +579,7 @@ Itf *create_theme_dialog(McsPlugin * mcs_plugin)
     dialog->button3 = gtk_button_new();
     gtk_button_set_label(GTK_BUTTON(dialog->button3), current_font);
     gtk_widget_show(dialog->button3);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame2), dialog->button3);
+    xfce_framebox_add(XFCE_FRAMEBOX (dialog->frame2), dialog->button3);
 
     dialog->frame3 = xfce_framebox_new (_("Toolbar Style"), TRUE);
     gtk_widget_show(dialog->frame3);
@@ -423,7 +587,7 @@ Itf *create_theme_dialog(McsPlugin * mcs_plugin)
 
     dialog->optionmenu1 = gtk_option_menu_new();
     gtk_widget_show(dialog->optionmenu1);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame3), dialog->optionmenu1);
+    xfce_framebox_add(XFCE_FRAMEBOX (dialog->frame3), dialog->optionmenu1);
 
     dialog->menu1 = gtk_menu_new();
 
@@ -492,14 +656,24 @@ static void setup_dialog(Itf * itf)
 
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(itf->theme_treeview), -1, NULL, gtk_cell_renderer_text_new(), "text", THEME_NAME_COLUMN, NULL);
     model = (GtkTreeModel *) gtk_list_store_new(N_COLUMNS, G_TYPE_STRING);
-    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), 0, sort_func, NULL, NULL);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), 0, sort_func, DEFAULT_THEME, NULL);
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 0, GTK_SORT_ASCENDING);
     gtk_tree_view_set_model(GTK_TREE_VIEW(itf->theme_treeview), model);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(itf->theme_treeview));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
     g_signal_connect(G_OBJECT(selection), "changed", (GCallback) theme_selection_changed, itf->mcs_plugin);
 
+    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(itf->icon_theme_treeview), -1, NULL, gtk_cell_renderer_text_new(), "text", THEME_NAME_COLUMN, NULL);
+    model = (GtkTreeModel *) gtk_list_store_new(N_COLUMNS, G_TYPE_STRING);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), 0, sort_func, DEFAULT_ICON_THEME, NULL);
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 0, GTK_SORT_ASCENDING);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(itf->icon_theme_treeview), model);
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(itf->icon_theme_treeview));
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+    g_signal_connect(G_OBJECT(selection), "changed", (GCallback) icon_theme_selection_changed, itf->mcs_plugin);
+
     read_themes(itf);
+    read_icon_themes(itf);
 
     g_signal_connect(G_OBJECT(itf->theme_dialog), "response", G_CALLBACK(cb_dialog_response), itf->mcs_plugin);
     g_signal_connect(G_OBJECT(itf->button3), "clicked", G_CALLBACK(show_font_selection), itf);
@@ -533,7 +707,7 @@ McsPluginInitResult mcs_plugin_init(McsPlugin * mcs_plugin)
     mcs_plugin->run_dialog = run_dialog;
     mcs_plugin->icon = inline_icon_at_size(default_icon_data, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
     mcs_manager_notify(mcs_plugin->manager, CHANNEL);
-    return (MCS_PLUGIN_INIT_OK);
+    return(MCS_PLUGIN_INIT_OK);
 }
 
 static void create_channel(McsPlugin * mcs_plugin)
@@ -545,8 +719,7 @@ static void create_channel(McsPlugin * mcs_plugin)
     mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL, rcfile);
     g_free(rcfile);
 
-    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Net/ThemeName",
-		    CHANNEL);
+    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Net/ThemeName", CHANNEL);
     if(setting)
     {
         if(current_theme)
@@ -562,10 +735,30 @@ static void create_channel(McsPlugin * mcs_plugin)
             g_free(current_theme);
         }
 
-        current_theme = g_strdup(DEFAULT_THEME);
+        current_theme = g_strdup(INITIAL_THEME);
         mcs_manager_set_string(mcs_plugin->manager, "Net/ThemeName", CHANNEL, current_theme);
     }
 
+    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Net/IconThemeName", CHANNEL);
+    if(setting)
+    {
+        if(current_icon_theme)
+        {
+            g_free(current_icon_theme);
+        }
+        current_icon_theme = g_strdup(setting->data.v_string);
+    }
+    else
+    {
+        if(current_icon_theme)
+        {
+            g_free(current_icon_theme);
+        }
+
+        current_icon_theme = g_strdup(DEFAULT_ICON_THEME);
+        mcs_manager_set_string(mcs_plugin->manager, "Net/IconThemeName", CHANNEL, current_theme);
+    }
+
     setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Gtk/FontName", CHANNEL);
     if(setting)
     {
@@ -607,30 +800,28 @@ static void create_channel(McsPlugin * mcs_plugin)
     }
 }
 
-static gboolean
-write_options(McsPlugin *mcs_plugin)
+static gboolean write_options(McsPlugin * mcs_plugin)
 {
     gboolean result;
     gchar *rcfile;
 
     rcfile = xfce_get_userfile(RCDIR, RCFILE, NULL);
-    result = mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL,
-		    rcfile);
+    result = mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL, rcfile);
     g_free(rcfile);
 
     return(result);
 }
 
-static void
-run_dialog(McsPlugin *mcs_plugin)
+static void run_dialog(McsPlugin * mcs_plugin)
 {
     Itf *dialog;
 
-    if (!is_running) {
-    	is_running = TRUE;
+    if (!is_running)
+    {
+        is_running = TRUE;
 
-    	dialog = create_theme_dialog(mcs_plugin);
-    	setup_dialog(dialog);
+        dialog = create_theme_dialog(mcs_plugin);
+        setup_dialog(dialog);
     }
 }
 
